//Check Resource Capacity v1.0

//Purpose:
//This function enforces daily limits on a resource established in the below config object
//e.g. The resource 'Jason Young' cannot have more than 8 events scheduled per day

//Action Type: Before Event save
//Prevent Default Action: Yes

//More info on Before Event Save actions and objects here:
//https://www.seedcode.com/pmwiki/index.php?n=DayBackOnline.CustomActionObjects#BeforeSave

//specify the limits for each resource
var config = {
  'Jason Young' : 8,
  'Tanner Ellen' : 8,
  'John Sindelar' : 12
};

//----------- You shouldn’t need to edit below this line -------------------

//call our function
CheckResourceCapacity(config);

function CheckResourceCapacity(config) {

  //gather values from event we're trying to save
  var resource = editEvent.resource[0];
  var start = editEvent.start.format('YYYY-MM-DD');
  var stop = editEvent.end.format('YYYY-MM-DD');

  //retrieve the limit from the config object
  var limit = config[resource];

  //retrieve values from context for SOQL query
  //specify the object name we're testing for capacity
  var objectName = event.schedule.objectName;
  var objectNameForDialog = objectName.replace('__c','');
  //retrieve Salesforce field names from our field mapping
  var fieldMap = event.schedule.fieldMap;
  var resourceField = fieldMap.resource;
  var startField = fieldMap.start;
  var endField = fieldMap.end;

  //new moment objects for start and stop
  var mStart = moment(moment(start).format('YYYY-MM-DD'));
  var mStop = moment(moment(stop).format('YYYY-MM-DD'));

  //retrieve our canvas client object for authentication
  var client = fbk.client();

  //retrieve our canvas context object for REST links/urls
  var context = fbk.context();

  //retrieve the query URL from context
  var url = context.links.queryUrl;

  //create SOQL statement
  var soql = 'SELECT+ID,' + startField + ',' + endField + '+FROM+' + objectName + '+WHERE+' + startField + '+<=+' + stop + '+AND+'+ endField + '>=+' + start + '+AND+' + resourceField + '+=+\'' + resource + '\'';
  //make sure this event isn't included in the query
  if (editEvent.eventID) {
      soql += '+AND+Id<>\'' + editEvent.eventID + '\'';
  }

  //final URL for GET
  var finalUrl = url + '?q=' + soql;

  //build settings object for Ajax call to Salesforce
  var settings = {};
  settings.client = client;
  settings.contentType = 'application/json';
  settings.success = callback;

  //Use canvas function to query
  Sfdc.canvas.client.ajax ( finalUrl , settings );

  function callback(data) {
    //existing spots retrieved for this resource
    if(data.status!==200){
      alert('Error looking up Events For This Resource');
      return;
    }
    var records = data.payload.records;
    //new moment object to represent each day in the range.
    var starting = mStart.clone();
    var dif = mStop.diff(starting);
    var errorMessage=[];
    var resultData = {};
    while (dif >= 0) {
      resultData[starting.format('YYYY-MM-DD')] = 0;
      for ( var i = 0 ; i < records.length ; i++){
        //loop through all records to see which ones spen this day.
        var thisRecordStart = moment(records[i][startField]);
        var thisRecordEnd = moment(records[i][endField]);
        //if this record occurs on this day, add to today bucket
        if(starting.isBetween(thisRecordStart,thisRecordEnd)||starting.isSame(thisRecordStart) || starting.isSame(thisRecordEnd)){
          resultData[starting.format('YYYY-MM-DD')]++;
        }
      }
      //if we're over capacity for this bucket, then create an error message of dates
      if( (resultData[starting.format('YYYY-MM-DD')] + 1 ) > limit) {
        errorMessage.push(starting.format('YYYY-MM-DD'));
      }
      //incrament our while loop
      starting.add(1,'days');
      dif = mStop.diff(starting);
    }
    if(errorMessage.length > 0) {
      //we're over the limit somewhere. Throw error and kill transaction
      utilities.showModal('Resource Over Limit',
       'This ' + objectNameForDialog + ' puts this resource over capacity on the following dates:\n' + errorMessage.join(', '),
        'OK', function () {
          action.callbacks.cancel();
        });
    }
    else {
      //not over the limit anywhere.
      action.callbacks.confirm();
    }
  }
}
